<?php

use CRM_FormProcessor_ExtensionUtil as E;

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
class CRM_FormProcessor_BAO_FormProcessorInput extends CRM_FormProcessor_DAO_FormProcessorInput {
  
	 /**
   * Function to get values
   * 
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $input = new CRM_FormProcessor_BAO_FormProcessorInput();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $input->$key = $value;
        }
      }
    }
    $input->find();
    while ($input->fetch()) {
      $row = array();
      self::storeValues($input, $row);
      if (!empty($row['form_processor_instance_id'])) {
      	$type = \Civi::service('form_processor_type_factory')->getTypeByName($row['type']);
				if ($type) {
					$configuration = $type->getDefaultConfiguration();
					if (isset($row['configuration']) && is_string($row['configuration'])) {
						$row['configuration'] = json_decode($row['configuration'], true);
					}
					if (empty($row['configuration']) || !is_array($row['configuration'])) {
						$row['configuration'] = array();
					}
					foreach($row['configuration'] as $name => $value) {
						$configuration->set($name, $value);
					}
					$type->setConfiguration($configuration);
					
					$row['type'] = $type;
				}
				
				$row['validators'] = array_values(CRM_FormProcessor_BAO_FormProcessorValidation::getValues(array('entity_id' => $input->id, 'entity' => 'FormProcessorInput')));
				
        $result[$row['id']] = $row;
      } else {
        //invalid input because no there is no form processor
        CRM_FormProcessor_BAO_FormProcessorInput::deleteWithId($row['id']);
      }
    }
    return $result;
  }

  /**
   * Function to add or update form processor input
   * 
   * @param array $params 
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a form processor input');
    }
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorInput', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorInput', NULL, $params);
    }
		
    $input = new CRM_FormProcessor_BAO_FormProcessorInput();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $input->$key = $value;
      }
    }
		if (is_array($input->configuration)) {
			$input->configuration = json_encode($input->configuration);
		}
    $input->save();
    self::storeValues($input, $result);
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorInput', $input->id, $input);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorInput', $input->id, $input);
    }
		
    return $result;
  }

  /**
   * Function to delete a form processor input with id
   * 
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor input');
    }
		
		CRM_Utils_Hook::pre('delete', 'FormProcessorInput', $id, CRM_Core_DAO::$_nullArray);
		
    $input = new CRM_FormProcessor_BAO_FormProcessorInput();
    $input->id = $id;
    if ($input->find(true)) {
    	CRM_FormProcessor_BAO_FormProcessorValidation::deleteWithEntityId('FormProcessorInput', $input->id);
      $input->delete();
    }
		
		CRM_Utils_Hook::post('delete', 'FormProcessorInput', $id, CRM_Core_DAO::$_nullArray);
		
    return;
  }

	/**
   * Function to delete all inputs with a form processor instance id
   *
   * @param int $formProcessorInstanceId
   * @access public
   * @static
   */
  public static function deleteWithFormProcessorInstanceId($formProcessorInstanceId) {
    $input = new CRM_FormProcessor_BAO_FormProcessorInput();
    $input->form_processor_instance_id = $formProcessorInstanceId;
    $input->find(FALSE);
    while ($input->fetch()) {
      CRM_Utils_Hook::pre('delete', 'FormProcessorInput', $input->id, CRM_Core_DAO::$_nullArray);
		
			CRM_FormProcessor_BAO_FormProcessorValidation::deleteWithEntityId('FormProcessorInput', $input->id);
      $input->delete();
			
			CRM_Utils_Hook::post('delete', 'FormProcessorInput', $input->id, CRM_Core_DAO::$_nullArray);
    }
  }
	
	
}